// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//  Giuseppe Vacanti (cosine science & computing bv)
//  March 11, 2003
// 
//   $Id: DataReader.h,v 1.1 2003/05/06 08:53:03 gvacanti Exp $
#ifndef DATAREADER_H
#define DATAREADER_H

#include<vector>
#include<SmartDal>
#include<cassert>

template<typename T> class DataReader {
public:
  DataReader() {}
  std::vector<T> & operator()(const SmartDal::ColumnH & c);
  std::vector<T> & data();
private:
  std::vector<T> v_;
};


template<typename T> std::vector<T> & DataReader<T>::operator()(const SmartDal::ColumnH & c) {
  assert(c->dimensions() == 1);
  assert(c->elements() == 1);
  SmartDal::ColumnDataH cd = c->data();
  size_t n = cd->count(); // in case the column had been 'seek-ed'
  v_.resize(n);
  // FIXME: Why do I need to qualify the method?
  //    T * pt = cd->data<T>();
  T * pt = cd->Data::data<T>();
  v_.assign(pt, pt + n);
  return v_;
}

template<typename T> std::vector<T> & DataReader<T>::data() {
  return v_;
}

template<> std::vector<std::string> & 
DataReader<std::string>::operator()(const SmartDal::ColumnH & c) {
  assert(c->dimensions() == 1);
  size_t n = c->count();
  v_.resize(n);
  unsigned offset = c->from();
  for(unsigned k = 0; k < n; ++k)
    v_[k] = SmartDal::get_string_row(c, k + offset);
  return v_;
}

#endif
