// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//  Giuseppe Vacanti (cosine science & computing bv)
//  March  3, 2003
//
//   $Id: TestTimeRows.cc,v 1.4 2003/11/17 13:33:57 jhoar Exp $
#include<SmartDal>
#include<TimeRows.h>
#include<STime.h>

#include<iostream>

#include<cassert>

void test1();
void test2();

int main() {
  
  test1();
  test2();

}

void test1() {
  using namespace SmartDal;
  std::string fname("TestTimeRows.ds");
  longVec dim;
  dim.push_back(19);
  
  DataSetH set = open_set(fname, Create);
  TableH tab = set->addTable("table", 10);
  ColumnH col = tab->addColumn("TIME", Dal::DString, "", "", dim);
  STime tstart("2002-01-01T00:00:00");
  double step = 25.;
  for(unsigned k = 0; k < tab->rows(); ++k){
    STime t = tstart + k*step;
    set_string_row(col, k, t.toString(0));
	std::cerr << t << std::endl;
  }

  TimeRows tr(tab);

  std::cerr << "lower bound start " << tr.index(tr.lower_bound(tstart))
	    << std::endl;
  std::cerr << "upper bound start " << tr.index(tr.upper_bound(tstart))
	    << std::endl;
  
  assert(tr.index(tr.lower_bound(tstart)) == 0);
  assert(tr.index(tr.upper_bound(tstart)) == 1);

  unsigned nrows = tab->rows();

  // lower_bound
  for(unsigned k = 0; k < nrows; ++k){
    // Look for the lower bound when t is 1s larger than the entries
    // present in the file. The question we ask here is: which is the
    // earliest possible row where the time find could be inserted and
    // preserve the sorting?
    STime find = tstart + k*step + 1.;
    unsigned expect = k + 1;

    if (tr.lower_bound(find) == tr.end())
	    std::cerr << "lower_bound " << find << " + " << k << " "
  		<< " End of iterator "
	    << std::endl;
	else 
	    std::cerr << "lower_bound " << find << " + " << k << " "
			<< tr.index(tr.lower_bound(find)) << "==" 
			<< tr.totime(tr.lower_bound(find))
	    	<< " expect " << expect 
		    << std::endl;     

    if( k != nrows -1)
      assert(tr.index(tr.lower_bound(find)) == expect);
    else
      assert(tr.lower_bound(find) == tr.end());
  }

  
  // Same as above, but now 1s earlier.
  for(unsigned k = 0; k < nrows; ++k){
    STime find = tstart + k*step - 1.;
    std::cerr << "lower_bound " << find << " - " << k << " "
	      << tr.index(tr.lower_bound(find))
	      << std::endl;
    assert(tr.index(tr.lower_bound(find)) == k);
  }

  // upper_bound Same as before, but now I ask what is the latest
  // place where find could be inserted preserving the ordeding.
  for(unsigned k = 0; k < nrows; ++k){
    STime find = tstart + k*step + 1.;
    unsigned expect = k+1;
	
    if (tr.upper_bound(find) == tr.end())
	    std::cerr << "lower_bound " << find << " + " << k << " "
  		<< " End of iterator "
	    << std::endl;
	else 
    	std::cerr << "upper_bound " << find << " + " << k << " "
	    	<< tr.index(tr.upper_bound(find)) << "==" 
			<< tr.totime(tr.upper_bound(find))
        	<< " expect " << expect
	    	<< std::endl;
    if( k != nrows -1)
      assert(tr.index(tr.upper_bound(find)) == expect);
    else
      assert(tr.lower_bound(find) == tr.end());
  }

  // And again as before.
  for(unsigned k = 0; k < nrows; ++k){
    STime find = tstart + k*step - 1.;
    std::cerr << "upper_bound " << find << " - " << k << " "
	      << tr.index(tr.upper_bound(find))
	      << std::endl;
    assert(tr.index(tr.upper_bound(find)) == k);
  }

   std::cerr << tr.totime(tr.begin()) << " " << tstart << std::endl;
   assert(tr.totime(tr.begin()) == tstart);
   std::cerr << tr.totime(++tr.begin()) << " " << tstart + 25. << std::endl;
   double diff = tr.totime(++tr.begin()) - (tstart + 25.);
   std::cerr << "Time difference: " << diff << std::endl;
   assert(tr.totime(++tr.begin()) - (tstart + 25.) < 1e-9);

   
   std::pair<long, long> p = tr.active_range(tstart + 2.);
   std::cerr << p.first << " " << p.second << std::endl;
   assert(p.first == 0);
   assert(p.second == 1);

   
   std::cerr << tr.totime(tr.active_record(tstart + 2.)) << std::endl;
   std::cerr << tr.index(tr.active_record(tstart + 2.)) << std::endl;

}

void test2() {
  using namespace SmartDal;
  std::string fname("TestTimeRows.ds");
  longVec dim;
  dim.push_back(19);
  
  DataSetH set = open_set(fname, Create);
  TableH tab = set->addTable("table", 10);
  ColumnH col = tab->addColumn("TIME", Dal::DString, "", "", dim);
  STime tstart("2002-01-01T00:00:00");
  for(unsigned k = 0; k < 5 ; ++k){
    set_string_row(col, k, tstart.toString(0));
	std::cerr << tstart << std::endl;
  }
  STime later = tstart + 3600.;
  for(unsigned k = 5; k < 10 ; ++k){
    set_string_row(col, k, later.toString(0));
	std::cerr << later << std::endl;
  }

  TimeRows tr(tab);

  std::cerr << "lower bound start " << tr.index(tr.lower_bound(tstart))
	    << std::endl;
  std::cerr << "upper bound start " << tr.index(tr.upper_bound(tstart))
	    << std::endl;
  

  // This we expect to deliver a reasonable range.
  std::cerr << tr.totime(tr.active_record(tstart + 2.))
	    << " at record "
	    << tr.index(tr.active_record(tstart + 2.)) << std::endl;
  std::pair<long, long> p = tr.active_range(tstart + 2.);
  std::cerr << p.first << " " << p.second << std::endl;
  assert(p.first == 0);
  assert(p.second == 5);

  // This should return tr.end()
  assert(tr.lower_bound(later + 2) == tr.end());
  assert(tr.upper_bound(later + 2) == tr.end());

  p = tr.active_range(later + 2.);
  std::cerr << p.first << " " << p.second << std::endl;

  assert(p.first == 5);
  assert(p.second == 10);
  
}
